
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtCore/QDir>
#include <QtCore/QCoreApplication>
#include <QtCore/QLocale>
#include <QtCore/QLibraryInfo>
#include <QtCore/QTranslator>
#include <QtCore/QProcess>
#include <QtCore/QtDebug>

#include <QtGui/QDesktopServices>
#include <QtGui/QDesktopWidget>
#include <QtGui/QApplication>
#include <QtGui/QPixmap>
#include <QtGui/QImageReader>

#include "MiscFunctions.h"
#include "LogHandler.h"
#include "AppSettings.h"

#if !defined(WIN32)
#include <unistd.h> // for unlink
#endif

QString MiscFunctions::getValidStorage()
{
#ifdef Q_WS_X11 // on unix => write to the user home
    return QDir::homePath() + "/." + PACKAGE_NAME;

#else // other case
    return QDesktopServices::storageLocation(QDesktopServices::DataLocation);

#endif
}

void MiscFunctions::cleanTempStorage(const QStringList &filter, const QString &root)
{
    LogHandler::getInstance()->reportDebug( QString("Cleaning temp storage %1 ...").arg( getValidStorage() ) );
    QDir dir( getValidStorage() );
    QFileInfoList list = dir.entryInfoList(filter);
    foreach (const QFileInfo &info, list)
    {
        if ( root.isEmpty() )
        {
            unlink( info.absoluteFilePath().toStdString().c_str() );
        }
        else // restrict unlink to file containing the "root" value
        {
            if ( info.absoluteFilePath().contains(root) )
            {
                unlink( info.absoluteFilePath().toStdString().c_str() );
                LogHandler::getInstance()->reportDebug( QString("Deleting %1").arg( info.absoluteFilePath() ) );
            }
        }
    }
}

QStringList MiscFunctions::getAvailablesImageFormatsList()
{
    QList<QByteArray> supportedFormats = QImageReader::supportedImageFormats();
    QStringList result;
    QListIterator<QByteArray> supportedFormat(supportedFormats);
    QByteArray format;

    while ( supportedFormat.hasNext() )
    {
        format = supportedFormat.next().toLower();

        if ( !result.contains(format) )
            result << format;
    }

    return result;
}

/*
 * Extract from Wally - Qt4 wallpaper/background changer
 * Copyright (C) 2009  Antonio Di Monaco <tony@becrux.com>
 */

QMap<QString, QString> MiscFunctions::getLongImageFormats()
{
    QMap<QString, QString> formats;

    formats["bw"] = "Black & White";
    formats["eps"] = "Encapsulated Postscript";
    formats["epsf"] = "Encapsulated PostScript";
    formats["epsi"] = "Encapsulated PostScript Interchange";
    formats["exr"] = "OpenEXR";
    formats["pcx"] = "PC Paintbrush Exchange";
    formats["psd"] = "Photoshop Document";
    formats["rgb"] = "Raw red, green, and blue samples";
    formats["rgba"] = "Raw red, green, blue, and alpha samples";
    formats["sgi"] = "Irix RGB";
    formats["tga"] = "Truevision Targa";
    formats["xcf"] = "eXperimental Computing Facility (GIMP)";
    formats["dds"] = "DirectDraw Surface";
    formats["xv"] = "Khoros Visualization";
    formats["bmp"] = "Windows Bitmap";
    formats["gif"] = "Graphic Interchange Format";
    formats["jpg"] = "Joint Photographic Experts Group";
    formats["jpeg"] = "Joint Photographic Experts Group";
    formats["jp2"] = "Joint Photographic Experts Group 2000";
    formats["mng"] = "Multiple-image Network Graphics";
    formats["png"] = "Portable Network Graphics";
    formats["pbm"] = "Portable Bitmap";
    formats["pgm"] = "Portable Graymap";
    formats["ppm"] = "Portable Pixmap";
    formats["tiff"] = "Tagged Image File Format";
    formats["tif"] = "Tagged Image File Format";
    formats["xbm"] = "X11 Bitmap";
    formats["xpm"] = "X11 Pixmap";
    formats["ico"] = "Icon Image";
    formats["svg"] = "Scalable Vector Graphics";

    return formats;
}

QString MiscFunctions::getAvailablesImageFormats()
{
    QString imglist;
    QStringList formats = getAvailablesImageFormatsList();
    for (int i = 0; i < formats.size(); ++i)
        imglist += "*." + formats[i] + " ";

    return imglist;
}

QString MiscFunctions::getTranslationsFile(const QString &lang)
{
    return QString("%1_%2.qm").arg(PACKAGE_NAME).arg(lang).toLower();
}

QString MiscFunctions::getTranslationsPath(const QString &refLang)
{
    // search in application path
    QStringList ldir;
    ldir << QCoreApplication::applicationDirPath () + "/lang";
    ldir << QCoreApplication::applicationDirPath () + "/../lang";
    ldir << QCoreApplication::applicationDirPath () + "/../../lang";
    ldir << QCoreApplication::applicationDirPath () + "/../translations"; // MacOSX
    ldir << QString("/usr/share/%1/locale").arg(PACKAGE_NAME).toLower();
    ldir << QString("/usr/local/share/%1/locale").arg(PACKAGE_NAME).toLower();
    ldir << "/usr/local/share/locale";
    ldir << "/usr/share/local/locale";
    ldir << "/usr/share/locale";

    QString ext;
    if ( !refLang.isEmpty() )
        ext = "/" + getTranslationsFile(refLang);

    foreach (const QString &dir, ldir)
    {
        if ( QFileInfo(dir + ext).exists() )
            return dir;
    }

    return "";
}

QMap<QString, QString> MiscFunctions::getAvailableLanguages()
{
    QMap<QString, QString> languageMap;
    QDir dir( MiscFunctions::getTranslationsPath("fr") );
    QRegExp expr( QString("^%1_(\\w+)\\.qm$").arg(PACKAGE_NAME).toLower() );
    QStringList files = dir.entryList(QDir::Files, QDir::Name);

    LogHandler::getInstance()->reportDebug( QString("translations path %1").arg( dir.path() ) );

    foreach ( const QString &file, files )
    {
        if ( !file.contains(expr) )
            continue;

        QString lang = expr.cap(1);
        QString name = QString("%1 (%2)").arg(QLocale::languageToString( QLocale(lang).language() ), lang);

        if ( lang.contains("_") ) // detect variant
        {
            QString country = QLocale::countryToString( QLocale(lang).country() );
            name = QString("%1 (%2) (%3)").arg(QLocale::languageToString( QLocale(lang).language() ), country, lang);
        }

        languageMap[name] = lang;
    }
    return languageMap;
}

void MiscFunctions::setDefaultLanguage()
{
    QString lang;
    AppSettings settings;

    settings.beginGroup("Application");
    lang = settings.value("currentLanguage","auto").toString();
    settings.endGroup();

    if ( lang.isEmpty() )
        lang = QLocale::system().name().left(2);

    if ( !lang.isEmpty() )
        setLanguage(lang);
}

void MiscFunctions::setLanguage(const QString& lang)
{
    QString language(lang);
    LogHandler::getInstance()->reportDebug( QString("setting language to: %1").arg(language) );

    // special cases
    if (language == "auto") // auto detection
    {
        language = QLocale::system().name().left(2);
    }
    else if (language == "default") // no use of translator
    {
        return;
    }

    // try load the qt translator for selected language
    QTranslator *qt = new QTranslator();
    QStringList excludedFiles;
    QString globalTranslationPath = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
    if ( !QFileInfo(globalTranslationPath).exists() )
        globalTranslationPath = MiscFunctions::getTranslationsPath("fr");

    if ( qt->load( "qt_" + language, globalTranslationPath ) )
    {
        LogHandler::getInstance()->reportDebug( QString("successfully loaded data from %1").arg( globalTranslationPath + "/qt_" + language ) );
        QCoreApplication::installTranslator(qt);
        excludedFiles << "qt_" + language + ".qm";
    }
    else
    {
        delete qt;
    }

    QString suff = language + ".qm";
    QDir dir( MiscFunctions::getTranslationsPath("fr") );

    foreach ( const QString &s, dir.entryList(QDir::Files | QDir::Readable) )
    {
        if ( !s.endsWith(suff) || excludedFiles.contains(s) )
            continue;
        QTranslator *t = new QTranslator();

        if ( t->load( dir.filePath(s) ) )
        {
            QCoreApplication::installTranslator(t);
            LogHandler::getInstance()->reportDebug( QString("successfully loaded data from %1").arg( dir.filePath(s) ) );
        }
        else
        {
            delete t;
            LogHandler::getInstance()->reportDebug( QString("failed to load data from %1").arg( dir.filePath(s) ) );
        }
    }
}

QPixmap MiscFunctions::getScreenShot(int screenNb)
{
    QDesktopWidget *scr = QApplication::desktop();
    return QPixmap::grabWindow( scr->screen(screenNb)->winId() );
}

QString MiscFunctions::getSignificantPart(const QString &file)
{
    // we send only the last dir and the basename of the file
    QStringList list = file.split( QRegExp("/") );

    if (list.size() >= 2)
        return list[list.size() - 2 ] + "/" + list[list.size() - 1];

    return list[list.size() - 1];
}

QString MiscFunctions::launchExternalSynchronousProcess(const QString program, const QStringList &arguments)
{
    QProcess proc;

    /* Start process */
    proc.setReadChannelMode(QProcess::MergedChannels);
    proc.start(program, arguments);
    proc.waitForFinished();

    /* Retrieve the output results and get the difference value */
    QString output = ( QString( proc.readAllStandardOutput() ) ).trimmed();
    proc.kill();
    proc.waitForFinished();
    return output;
}

void MiscFunctions::updateApplicationIdentity()
{
    QCoreApplication::setApplicationName(PACKAGE_NAME);
    QCoreApplication::setApplicationVersion(PACKAGE_VERSION);
    QCoreApplication::setOrganizationName(PACKAGE_ORGANIZATION);
}

bool MiscFunctions::hasSmallUI()
{
    QRect screen = QApplication::desktop()->screenGeometry();

    // A bit ugly, no ? TODO find a better way to define a "small" screen
    if (screen.height() < 400 * 2)
        return true;
    else
        return false;
}

void MiscFunctions::setRunOnStartup(bool runOnStartup)
{
#ifdef Q_OS_WIN

    QSettings regSettings("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", QSettings::NativeFormat);

    if (runOnStartup)
        regSettings.setValue( PACKAGE_NAME,QDir::toNativeSeparators( QCoreApplication::applicationFilePath () ) );
    else
        regSettings.remove(PACKAGE_NAME);
#endif
#ifdef Q_OS_LINUX
    QString desktopFileContents = QString("[Desktop Entry]\n"
                                          "Name=%1\n"
                                          "Comment=%2\n"
                                          "Icon=%3\n"
                                          "Type=Application\n"
                                          "Exec=%4\n"
                                          "Hidden=false\n"
                                          "NoDisplay=false\n"
                                          "X-GNOME-Autostart-enabled=true\n")
                                      .arg(PACKAGE_NAME)
                                      .arg(PACKAGE_DESCRIPTION)
                                      .arg( QString(PACKAGE_NAME).toLower() )
                                      .arg( QDir::toNativeSeparators( QCoreApplication::applicationFilePath() ) );

    QString autostartLocation;
    char* xgdConfigHome = getenv("XDG_CONFIG_HOME");
    if(xgdConfigHome != NULL)
        autostartLocation = QString(QString(xgdConfigHome) + "/.config/autostart");
    else
        autostartLocation = QString(QDir::homePath() + "/.config/autostart");

    QDir d;
    d.mkpath(autostartLocation);
    QFile autostartFile( autostartLocation + QString("/%1.desktop").arg( QString(PACKAGE_NAME).toLower() ) );
    if(runOnStartup)
    {
        if( !autostartFile.exists() )
        {
            qDebug() << "Installing autostart file to " << autostartLocation;
            autostartFile.open(QFile::WriteOnly);
            autostartFile.write( desktopFileContents.toLocal8Bit() );
            autostartFile.close();
        }
        autostartFile.setPermissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner | QFile::ReadGroup | QFile::WriteGroup | QFile::ReadOther);
    }
    else
    {
        if( autostartFile.exists() )
        {
            qDebug() << "Removing autostart file " << autostartFile.fileName();
            autostartFile.remove();
        }
    }
#endif
}
