
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtCore/QTimer>
#include <QtNetwork/QHostInfo>
#include <QtNetwork/QUdpSocket>

#include "NetworkManager.h"
#include "LogHandler.h"
#include "AppSettings.h"

#include "TcpClient.h"
#include "TcpServer.h"

const int defaultPort = 55555;
const int defaultPingTimeout = 15 * 1000;

NetworkManager::NetworkManager(QObject *parent)
    : QObject(parent),
    m_port(defaultPort),
    m_mode(MODE_UNCONNECTED),
    m_initialized(false),
    m_client(0),
    m_server(0),
    m_started(false)
{
    loadSettings();

    // can't initialize other member because current thread isn't final thread !!!
}

NetworkManager::~NetworkManager()
{
    delete m_client;
    delete m_server;
}

void NetworkManager::initialize()
{
    if (m_initialized)
        return;

    m_client = new TcpClient();

    connect( m_client,SIGNAL( stateChanged(QAbstractSocket::SocketState) ),this,SLOT( clientConnectionChanged(QAbstractSocket::SocketState) ) );
    connect( m_client,SIGNAL( newNetworkMessage(const NetworkMessage &) ),this,SIGNAL( newMessage(const NetworkMessage &) ) );

    m_server = new TcpServer();

    connect( m_server,SIGNAL( stateChanged(quint8) ),this,SLOT( serverConnectionChanged(quint8) ) );

    m_initialized = true;
}

void NetworkManager::stop()
{
    if (!m_initialized)
        return;

    m_client->stop();
    m_server->stop();
    m_started = false;
}

void NetworkManager::start()
{
    if (m_started)
        stop();
    m_started = true;
    initialize();

    updateConfig();
    if (m_mode == MODE_SERVER)
        m_server->start();
    else if (m_mode == MODE_CLIENT)
        m_client->start();
}

void NetworkManager::loadSettings()
{
    AppSettings settings;

    settings.beginGroup("NetworkManager");

    m_port = settings.value("udpPort",defaultPort).toInt();
    m_serverName = settings.value("serverName","").toString();
    m_mode = settings.value("mode",MODE_UNCONNECTED).toInt();

    settings.endGroup();
}

void NetworkManager::saveSettings()
{
    AppSettings settings;

    settings.beginGroup("NetworkManager");

    settings.setValue("udpPort", m_port);
    settings.setValue("serverName", m_serverName);
    settings.setValue("mode", m_mode);

    settings.endGroup();
}

void NetworkManager::setServerName(const QString &name)
{
    m_serverName = name;
}

void NetworkManager::updateConfig()
{
    if (m_client)
    {
        m_client->setPort(m_port);
        m_client->setHost(m_serverName);
    }

    if (m_server)
        m_server->setPort(m_port);
}

void NetworkManager::setPort(int p)
{
    if (p == m_port)
        return;

    m_port = p;

    updateConfig();
}

void NetworkManager::setMode(int mode)
{
    qDebug() << "NetworkManager::setMode " << mode;
    if (m_mode == mode)
        return;

    m_mode = mode;

    // clear client list
    //  if (m_mode == MODE_CLIENT)
    //    m_clients.clear();

    updateConfig();
}

void NetworkManager::sendNewMessage(const NetworkMessage &mess)
{
    if (m_mode == MODE_SERVER)
        m_server->sendNetworkMessage(mess);
    else if (m_mode == MODE_CLIENT)
        m_client->sendNetworkMessage(mess);
}

void NetworkManager::sendNewMessage(const QString &img)
{
    m_currentMessage = img;
}

QAbstractSocket::SocketState NetworkManager::getState() const
{
    if (m_mode == MODE_SERVER)
    {
        if (m_server->getState() == TcpServer::INACTIVE)
            return QAbstractSocket::UnconnectedState;
        else
            return QAbstractSocket::ConnectedState;
    }
    else if (m_mode == MODE_CLIENT)
    {
        return m_client->getState();
    }

    return QAbstractSocket::UnconnectedState;
}

QString NetworkManager::getStateAsString() const
{
    if (m_mode == MODE_SERVER)
        return m_server->getStateAsString( m_server->getState() );
    else if (m_mode == MODE_CLIENT)
    {
        return m_client->stateToString( m_client->getState() );
    }
    return tr("Idle");
}

void NetworkManager::clientConnectionChanged(QAbstractSocket::SocketState state)
{
    if (state == QAbstractSocket::ConnectedState)
        emit serverAlive();
    else if (state == QAbstractSocket::UnconnectedState)
        emit serverDown();

    emit connectionChanged();
}

void NetworkManager::serverConnectionChanged(quint8 )
{
    emit connectionChanged();
}

int NetworkManager::getClientNb() const
{
    return m_server->getConnectionNumber();
}
