
/******************************************************************************
   SyncWall: Wallpaper manager
   Copyright(C) 2011-2013  xbee@xbee.net

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/

#include <QtNetwork/QHostInfo>
#include <QtNetwork/QNetworkInterface>

#include "NetworkManagerConfigWidget.h"
#include "NetworkManager.h"
#include "ThreadPool.h"

#include "ui_NetworkManagerConfigWidget.h"

NetworkManagerConfigWidget::NetworkManagerConfigWidget(QWidget *parent)
    : QWidget(parent),
    m_ui(new Ui::NetworkManagerConfigWidget)
{
    m_ui->setupUi(this);

    m_manager = new NetworkManager();

    m_manager->moveToThread( ThreadPool::getInstance()->getThread() );

    // transmit NetworkManager signals
    connect( m_manager, SIGNAL( serverAlive() ), this, SIGNAL( newState() ) );
    connect( m_manager, SIGNAL( serverAlive() ), this, SLOT( serverConnected() ) );
    connect( m_manager, SIGNAL( serverDown() ), this, SIGNAL( newState() ) );
    connect( m_manager, SIGNAL( serverDown() ), this, SLOT( serverDisconnected() ) );
    connect( m_manager, SIGNAL( newMessage(const QString &) ), this, SIGNAL( newMessage(const QString &) ) );
    connect( m_manager, SIGNAL( newMessage(const NetworkMessage &) ), this, SIGNAL( newMessage(const NetworkMessage &) ) );
    connect( m_manager, SIGNAL( connectionChanged() ), this, SLOT( connectionChanged() ) );

    connect( this, SIGNAL( sigSetPort(int) ), m_manager, SLOT( setPort(int) ) );
    connect( this, SIGNAL( sigSetServerName(const QString &) ), m_manager, SLOT( setServerName(const QString &) ) );
    connect( this, SIGNAL( sigSetMode(int) ), m_manager, SLOT( setMode(int) ) );
    connect( this, SIGNAL( sigSendNewMessage(const QString &) ), m_manager, SLOT( sendNewMessage(const QString &) ) );
    connect( this, SIGNAL( sigSendNewMessage(const NetworkMessage &) ), m_manager, SLOT( sendNewMessage(const NetworkMessage &) ) );
    connect( this, SIGNAL( sigSaveSetting() ), m_manager, SLOT( saveSettings() ) );
    connect( this, SIGNAL( sigStart() ), m_manager, SLOT( start() ) );
    connect( this, SIGNAL( sigStop() ), m_manager, SLOT( stop() ) );

    updateUI();
}

NetworkManagerConfigWidget::~NetworkManagerConfigWidget()
{
    delete m_manager;
    delete m_ui;
}

void NetworkManagerConfigWidget::updateUI()
{
    m_ui->groupBoxShareImage->setChecked(m_manager->getMode() != NetworkManager::MODE_UNCONNECTED);
    m_ui->groupBoxServer->setChecked(m_manager->getMode() == NetworkManager::MODE_CLIENT);
    m_ui->labelClientNb->setEnabled(m_manager->getMode() == NetworkManager::MODE_SERVER);
    m_ui->labelClient->setEnabled(m_manager->getMode() == NetworkManager::MODE_SERVER);
    m_ui->lineEditServerName->setText( m_manager->getServerName() );
    m_ui->lineEditServerPort->setText( QString::number( m_manager->getPort() ) );

    // Update UI
    QString ip;
    foreach ( const QHostAddress &address, QNetworkInterface::allAddresses() )
    {
        if ( address.protocol() == QAbstractSocket::IPv4Protocol && address != QHostAddress(QHostAddress::LocalHost && address.toString().section( ".",-1,-1 ) != "1") )
        {
            ip = address.toString();
            break;
        }
    }
    m_ui->labelHostname->setText( QString("%1 (%2)").arg( QHostInfo::localHostName() ).arg(ip) );

    if (m_manager->getMode() != NetworkManager::MODE_UNCONNECTED)
        emit sigStart();
}

void NetworkManagerConfigWidget::on_pushButtonTestConnection_pressed()
{
    // QString serverName = m_ui->lineEditServerName->text();
    // int udpPort = m_ui->lineEditServerPort->text().toInt();

    //   if (serverName.isEmpty() || !udpPort)
    //     return;

    //emit sigSetPort(udpPort);

    // clear the previous text
    // m_ui->labelTestConnection->setText("");
    emit sigStart();
}

void NetworkManagerConfigWidget::on_lineEditServerName_editingFinished()
{
    // change color to valid modification
    QPalette pal;
    pal.setColor(QPalette::Text, Qt::black);
    m_ui->lineEditServerName->setPalette(pal);

    emit sigSetServerName( m_ui->lineEditServerName->text() );
}

void NetworkManagerConfigWidget::on_lineEditServerPort_editingFinished()
{
    // change color to valid modification
    QPalette pal;
    pal.setColor(QPalette::Text, Qt::black);
    m_ui->lineEditServerPort->setPalette(pal);

    emit sigSetPort( m_ui->lineEditServerPort->text().toInt() );
}

void NetworkManagerConfigWidget::serverConnected()
{
    m_ui->labelTestConnection->setText( tr("Server %1 reached ...").arg( m_manager->getServerName() ) );
    emit synchronizeByNetwork(true);
}

void NetworkManagerConfigWidget::serverDisconnected()
{
    m_ui->labelTestConnection->setText( tr("Disconnection from server %1 ...").arg( m_manager->getServerName() ) );
}

void NetworkManagerConfigWidget::on_groupBoxServer_clicked(bool val)
{
    bool synchroServer = val;

    m_ui->labelClientNb->setEnabled(!synchroServer);
    m_ui->labelClient->setEnabled(!synchroServer);

    // update the network manager
    if (synchroServer)
    {
        emit sigSetMode(NetworkManager::MODE_CLIENT);
        on_pushButtonTestConnection_pressed(); // check connection with server
    }
    else
    {
        emit sigSetMode(NetworkManager::MODE_SERVER);
    }
    emit sigStart();
    emit synchronizeByNetwork(val);
    emit newState();
}

void NetworkManagerConfigWidget::on_lineEditServerName_textEdited(const QString & /*text*/)
{
    // change color to highlight modification
    QPalette pal;
    pal.setColor(QPalette::Text, Qt::red);
    m_ui->lineEditServerName->setPalette(pal);
}

void NetworkManagerConfigWidget::on_lineEditServerPort_textEdited(const QString & /*text*/)
{
    // change color to highlight modification
    QPalette pal;
    pal.setColor(QPalette::Text, Qt::red);
    m_ui->lineEditServerPort->setPalette(pal);
}

void NetworkManagerConfigWidget::sendNewMessage(const NetworkMessage &mess)
{
    emit sigSendNewMessage(mess);
}

void NetworkManagerConfigWidget::sendNewMessage(const QString &mess)
{
    emit sigSendNewMessage(mess);
}

void NetworkManagerConfigWidget::saveSettings()
{
    m_manager->saveSettings();
}

int NetworkManagerConfigWidget::getNetworkMode() const
{
    return m_manager->getMode();
}

int NetworkManagerConfigWidget::getClientNb() const
{
    return m_manager->getClientNb();
}

QAbstractSocket::SocketState NetworkManagerConfigWidget::getState() const
{
    return m_manager->getState();
}

void NetworkManagerConfigWidget::startDeprecated()
{
    emit sigStart();
}

void NetworkManagerConfigWidget::on_groupBoxShareImage_toggled(bool val)
{
    if (val)
    {
        if ( m_ui->groupBoxServer->isChecked() )
        {
            emit synchronizeByNetwork(true);
            emit sigSetMode(NetworkManager::MODE_CLIENT);
        }
        else
        {
            emit synchronizeByNetwork(false);
            emit sigSetMode(NetworkManager::MODE_SERVER);
        }
        emit sigStart();
    }
    else
    {
        emit sigSetMode(NetworkManager::MODE_UNCONNECTED);
        emit sigStop();
        emit synchronizeByNetwork(false);
    }
    emit newState();
}

void NetworkManagerConfigWidget::connectionChanged()
{
    m_ui->labelClientNb->setText( QString::number( getClientNb() ) );
    m_ui->labelTestConnection->setText( m_manager->getStateAsString() );
    emit newState();
}
